Package: readBrukerFlexData
Version: 1.9.3
Date: 2024-10-02
Title: Reads Mass Spectrometry Data in Bruker *flex Format
Authors@R: 
    c(
        person(
            "Sebastian", "Gibb", role = c("aut", "cre"),
            email = "mail@sebastiangibb.de",
            comment = c(ORCID = "0000-0001-7406-4443")
        ),
        person("Samuel", "Granjeaud", role = "ctb"),
        person("Alan", "Race", role = "ctb",
            email = "alan.race@uni-marburg.de",
            comment = c(ORCID = "0000-0001-8996-2641")
        )
    )
Depends: R (>= 3.3.0)
Suggests: testthat
Description: Reads data files acquired by Bruker Daltonics' matrix-assisted
    laser desorption/ionization-time-of-flight mass spectrometer of the *flex
    series.
License: GPL (>= 3)
URL: https://strimmerlab.github.io/software/maldiquant/,
        https://github.com/sgibb/readBrukerFlexData/
BugReports: https://github.com/sgibb/readBrukerFlexData/issues/
LazyLoad: yes
RoxygenNote: 7.3.1
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2024-10-02 21:01:19 UTC; sebastian
Author: Sebastian Gibb [aut, cre] (<https://orcid.org/0000-0001-7406-4443>),
  Samuel Granjeaud [ctb],
  Alan Race [ctb] (<https://orcid.org/0000-0001-8996-2641>)
Maintainer: Sebastian Gibb <mail@sebastiangibb.de>
Repository: CRAN
Date/Publication: 2024-10-02 21:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 21:32:22 UTC; unix
