## Copyright 2010-2012 Sebastian Gibb
## <mail@sebastiangibb.de>
##
## This file is part of readBrukerFlexData for R and related languages.
##
## readBrukerFlexData is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## readBrukerFlexData is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with readBrukerFlexData. If not, see <https://www.gnu.org/licenses/>

#' Mass spectrum generated by Bruker Daltonics CompassXport
#'
#' This dataset was generated by Bruker Daltonics CompassXport and imported by
#' \code{\link[readMzXmlData]{readMzXmlFile}} to R. It is only needed for
#' comparison between Bruker Daltonics' HPC and
#' \code{\link[readBrukerFlexData]{.hpc}}.
#'
#' @format A list containing a mass and an intensity vector.
#' @source Examples/hpc/mzXML/hpc.mzXML
#' @seealso
#'  \code{\link[readBrukerFlexData]{.hpc}},
#'  \code{\link[readMzXmlData]{readMzXmlFile}}
#' @name cpSpecHpcMzXml
#' @rdname cpSpecHpcMzXml
#' @docType data
#' @keywords datasets
NULL
